@@ -45,6 +45,7 @@ struct connman_device {
 	connman_bool_t carrier;
 	connman_bool_t scanning;
 	connman_bool_t disconnected;
+	connman_bool_t reconnect;
 	connman_uint16_t scan_interval;
 	char *name;
 	char *node;
@@ -106,6 +107,14 @@ static void reset_scan_trigger(struct connman_device *device)
 	}
 }
 
+static void force_scan_trigger(struct connman_device *device)
+{
+	clear_scan_trigger(device);
+
+	device->scan_timeout = g_timeout_add_seconds(5,
+					device_scan_trigger, device);
+}
+
 static const char *type2description(enum connman_device_type type)
 {
 	switch (type) {
@@ -199,7 +208,7 @@ static int set_connected(struct connman_device *device,
 
 		switch (device->element.ipv4.method) {
 		case CONNMAN_IPCONFIG_METHOD_UNKNOWN:
-		case CONNMAN_IPCONFIG_METHOD_OFF:
+		case CONNMAN_IPCONFIG_METHOD_IGNORE:
 			return 0;
 		case CONNMAN_IPCONFIG_METHOD_STATIC:
 			type = CONNMAN_ELEMENT_TYPE_IPV4;
@@ -300,8 +309,6 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 	if (device->powered_pending == powered)
 		return -EALREADY;
 
-	device->powered_pending = powered;
-
 	if (!driver)
 		return -EINVAL;
 
@@ -309,12 +316,16 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 
 	if (powered == TRUE) {
 		if (driver->enable) {
+			device->powered_pending = powered;
+
 			err = driver->enable(device);
 			if (err == 0)
 				__connman_notifier_enable(type);
 		} else
 			err = -EINVAL;
 	} else {
+		device->powered_pending = powered;
+
 		clear_scan_trigger(device);
 
 		g_hash_table_remove_all(device->networks);
@@ -331,7 +342,9 @@ static int set_powered(struct connman_device *device, connman_bool_t powered)
 
 	if (err == 0) {
 		device->powered = powered;
-		powered_changed(device);
+
+		if (device->registered == TRUE)
+			powered_changed(device);
 	}
 
 	return err;
@@ -542,6 +555,9 @@ static DBusMessage *set_property(DBusConnection *conn,
 	} else if (g_str_has_prefix(name, "IPv4.") == TRUE) {
 		int err;
 
+		if (device->ipconfig == NULL)
+			return __connman_error_invalid_property(msg);
+
 		switch (device->mode) {
 		case CONNMAN_DEVICE_MODE_UNKNOWN:
 		case CONNMAN_DEVICE_MODE_NETWORK_SINGLE:
@@ -860,7 +876,10 @@ static void device_destruct(struct connman_element *element)
 	g_free(device->control);
 	g_free(device->interface);
 
-	connman_ipconfig_unref(device->ipconfig);
+	if (device->ipconfig != NULL) {
+		connman_ipconfig_unref(device->ipconfig);
+		device->ipconfig = NULL;
+	}
 
 	g_free(device->last_network);
 
@@ -942,12 +961,6 @@ struct connman_device *connman_device_create(const char *node,
 		break;
 	}
 
-	device->ipconfig = connman_ipconfig_create();
-	if (device->ipconfig == NULL) {
-		connman_device_unref(device);
-		return NULL;
-	}
-
 	device->networks = g_hash_table_new_full(g_str_hash, g_str_equal,
 						g_free, unregister_network);
 
@@ -1027,6 +1040,15 @@ const char *connman_device_get_path(struct connman_device *device)
 void connman_device_set_index(struct connman_device *device, int index)
 {
 	device->element.index = index;
+
+	if (device->ipconfig != NULL) {
+		if (index == connman_ipconfig_get_index(device->ipconfig))
+			return;
+
+		connman_ipconfig_unref(device->ipconfig);
+	}
+
+	device->ipconfig = connman_ipconfig_create(index);
 }
 
 /**
@@ -1219,6 +1241,8 @@ int __connman_device_set_blocked(struct connman_device *device,
 	if (device->offlinemode == TRUE)
 		return 0;
 
+	connman_info("%s {rfkill} blocked %d", device->interface, blocked);
+
 	if (blocked == FALSE)
 		powered = device->powered_persistent;
 	else
@@ -1264,6 +1288,8 @@ int __connman_device_scan(struct connman_device *device)
 	if (device->powered == FALSE)
 		return -ENOLINK;
 
+	reset_scan_trigger(device);
+
 	return device->driver->scan(device);
 }
 
@@ -1417,6 +1443,48 @@ void __connman_device_cleanup_networks(struct connman_device *device)
 					remove_unavailable_network, NULL);
 }
 
+static void scanning_changed(struct connman_device *device)
+{
+	DBusMessage *signal;
+	DBusMessageIter entry, value;
+	const char *key = "Scanning";
+
+	signal = dbus_message_new_signal(device->element.path,
+				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
+	if (signal == NULL)
+		return;
+
+	dbus_message_iter_init_append(signal, &entry);
+
+	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
+
+	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
+					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
+	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN,
+							&device->scanning);
+	dbus_message_iter_close_container(&entry, &value);
+
+	g_dbus_send_message(connection, signal);
+}
+
+static void mark_network_available(gpointer key, gpointer value,
+						gpointer user_data)
+{
+	struct connman_network *network = value;
+
+	connman_network_set_available(network, TRUE);
+}
+
+void connman_device_cleanup_scanning(struct connman_device *device)
+{
+	device->scanning = FALSE;
+
+	scanning_changed(device);
+
+	g_hash_table_foreach(device->networks,
+				mark_network_available, NULL);
+}
+
 /**
  * connman_device_set_scanning:
  * @device: device structure
@@ -1427,10 +1495,6 @@ void __connman_device_cleanup_networks(struct connman_device *device)
 int connman_device_set_scanning(struct connman_device *device,
 						connman_bool_t scanning)
 {
-	DBusMessage *signal;
-	DBusMessageIter entry, value;
-	const char *key = "Scanning";
-
 	DBG("device %p scanning %d", device, scanning);
 
 	if (!device->driver || !device->driver->scan)
@@ -1441,21 +1505,7 @@ int connman_device_set_scanning(struct connman_device *device,
 
 	device->scanning = scanning;
 
-	signal = dbus_message_new_signal(device->element.path,
-				CONNMAN_DEVICE_INTERFACE, "PropertyChanged");
-	if (signal == NULL)
-		return 0;
-
-	dbus_message_iter_init_append(signal, &entry);
-
-	dbus_message_iter_append_basic(&entry, DBUS_TYPE_STRING, &key);
-
-	dbus_message_iter_open_container(&entry, DBUS_TYPE_VARIANT,
-					DBUS_TYPE_BOOLEAN_AS_STRING, &value);
-	dbus_message_iter_append_basic(&value, DBUS_TYPE_BOOLEAN, &scanning);
-	dbus_message_iter_close_container(&entry, &value);
-
-	g_dbus_send_message(connection, signal);
+	scanning_changed(device);
 
 	if (scanning == TRUE) {
 		reset_scan_trigger(device);
@@ -1505,6 +1555,9 @@ int connman_device_set_disconnected(struct connman_device *device,
 
 	device->disconnected = disconnected;
 
+	if (disconnected == TRUE)
+		force_scan_trigger(device);
+
 	return 0;
 }
 
@@ -1738,6 +1791,18 @@ void __connman_device_set_network(struct connman_device *device,
 	}
 }
 
+void __connman_device_set_reconnect(struct connman_device *device,
+						connman_bool_t reconnect)
+{
+	device->reconnect = reconnect;
+}
+
+connman_bool_t  __connman_device_get_reconnect(
+				struct connman_device *device)
+{
+	return device->reconnect;
+}
+
 /**
  * connman_device_register:
  * @device: device structure
@@ -1748,7 +1813,7 @@ int connman_device_register(struct connman_device *device)
 {
 	__connman_storage_load_device(device);
 
-	device->offlinemode = __connman_manager_get_offlinemode();
+	device->offlinemode = __connman_profile_get_offlinemode();
 
 	return connman_element_register(&device->element, NULL);
 }
@@ -1857,6 +1922,7 @@ static struct connman_driver device_driver = {
 
 static int device_load(struct connman_device *device)
 {
+	const char *ident = __connman_profile_active_ident();
 	GKeyFile *keyfile;
 	GError *error = NULL;
 	gchar *identifier;
@@ -1865,7 +1931,7 @@ static int device_load(struct connman_device *device)
 
 	DBG("device %p", device);
 
-	keyfile = __connman_storage_open();
+	keyfile = __connman_storage_open(ident);
 	if (keyfile == NULL)
 		return 0;
 
@@ -1896,19 +1962,20 @@ static int device_load(struct connman_device *device)
 done:
 	g_free(identifier);
 
-	__connman_storage_close(keyfile, FALSE);
+	__connman_storage_close(ident, keyfile, FALSE);
 
 	return 0;
 }
 
 static int device_save(struct connman_device *device)
 {
+	const char *ident = __connman_profile_active_ident();
 	GKeyFile *keyfile;
 	gchar *identifier;
 
 	DBG("device %p", device);
 
-	keyfile = __connman_storage_open();
+	keyfile = __connman_storage_open(ident);
 	if (keyfile == NULL)
 		return 0;
 
@@ -1934,7 +2001,7 @@ static int device_save(struct connman_device *device)
 done:
 	g_free(identifier);
 
-	__connman_storage_close(keyfile, TRUE);
+	__connman_storage_close(ident, keyfile, TRUE);
 
 	return 0;
 }
